@@ -91,9 +91,35 @@ def parse_arguments():
                         help='Cost estimator user ID (provided when you signed up for the service)')
     parser.add_argument('--server-url', type=str, default=None,
                         help='Custom Spark History Server URL (e.g., http://localhost:18080)')
-    parser.add_argument('--save-local', action='store_true',
-                        help=argparse.SUPPRESS)
-    return parser.parse_args()
+    parser.add_argument('--batch-size', type=int, default=50,
+                        help='Number of stages to process in each batch (default: 50)')
+    parser.add_argument('--live-app', action='store_true',
+                        help='Indicate this is a live/running application (enables reverse processing and stage validation to handle race conditions)')
+    
+    args = parser.parse_args()
+    
+    if not args.live_app:
+        try:
+            import configparser
+            config = configparser.ConfigParser()
+            config_paths = ["configs/config.ini"]
+            
+            for path in config_paths:
+                if os.path.exists(path):
+                    config.read(path)
+                    if 'processing' in config and 'live_app' in config['processing']:
+                        try:
+                            args.live_app = config.getboolean('processing', 'live_app')
+                            logging.debug(f"Live app setting from config: {args.live_app}")
+                        except (ValueError, configparser.Error):
+                            logging.warning("Invalid live_app setting in config, using default: False")
+                            args.live_app = False
+                    break
+        except Exception as e:
+            logging.debug(f"Could not read live_app from config: {str(e)}")
+            args.live_app = False
+    
+    return args
 
 def upload_output(data: dict, upload_url: str) -> bool:
     """Upload JSON data to the specified URL using gzip compression."""
@@ -245,7 +271,7 @@ def process_app_id(app_id, applications, client, spark_stage_analyzer, s3_upload
     batch_filenames = []
     proceed_with_processing = True
 
-    if s3_upload_enabled and len(stages) > 0 and not (args and getattr(args, 'save_local', False)):
+    if s3_upload_enabled and len(stages) > 0 and not os.environ.get('SPARK_ANALYZER_SAVE_LOCAL_MODE') == '1':
         batch_size = 1000
         total_batches = (len(stages) + batch_size - 1) // batch_size
         print(f"🔄 Preparing {total_batches} batches for S3 upload (up to {batch_size} stages each)...")
@@ -302,21 +328,103 @@ def process_app_id(app_id, applications, client, spark_stage_analyzer, s3_upload
         "stages": []  
     }
 
+    if args.live_app:
+        stages_sorted = sorted(stages, key=lambda x: int(x.get('stageId', 0)), reverse=True)
+        print(f"🔄 Processing {len(stages_sorted)} stages in reverse order...")
+        
+        BATCH_SIZE = min(args.batch_size, 50)  
+        print(f"🔄 Using batch size {BATCH_SIZE} for live app processing...")
+    else:
+        stages_sorted = sorted(stages, key=lambda x: int(x.get('stageId', 0)))
+        print(f"🔄 Processing {len(stages_sorted)} stages in normal order (completed app)...")
+        
+        BATCH_SIZE = max(args.batch_size, 100)  
+        print(f"🔄 Using batch size {BATCH_SIZE} for completed app processing...")
+    
+    total_batches = (len(stages_sorted) + BATCH_SIZE - 1) // BATCH_SIZE
+    
     failed_stages = 0
-    for stage in tqdm.tqdm(stages, desc="Processing stages", unit="stage", dynamic_ncols=True, leave=True):
-        try:
-            stage_data = spark_stage_analyzer.format_stage_for_proto(stage, app_id, executor_metrics)
-            if stage_data:
-                proto_output["stages"].append(stage_data)
-        except Exception as e:
-            failed_stages += 1
-            logging.debug(f"Error processing stage {stage.get('stageId')}: {str(e)}")
+    skipped_stages = 0
+    
+    with tqdm.tqdm(total=len(stages_sorted), desc="Processing stages", unit="stage", dynamic_ncols=True, leave=True) as pbar:
+        for batch_idx in range(total_batches):
+            start_idx = batch_idx * BATCH_SIZE
+            end_idx = min(start_idx + BATCH_SIZE, len(stages_sorted))
+            batch_stages = stages_sorted[start_idx:end_idx]
+            
+            for stage in batch_stages:
+                try:
+                    stage_id = stage.get('stageId')
+                    if stage_id is None:
+                        logging.debug("Found stage with missing stageId, skipping")
+                        skipped_stages += 1
+                        pbar.update(1)
+                        continue
+                    
+                    if args.live_app:
+                        stage_exists = client.check_stage_exists(app_id, stage_id)
+                        if not stage_exists:
+                            logging.info(f"Stage {stage_id} not found (likely removed), skipping")
+                            skipped_stages += 1
+                            pbar.update(1)
+                            continue
+                        else:
+                            logging.debug(f"Stage {stage_id} validation passed")
+                    
+                    stage_data = spark_stage_analyzer.format_stage_for_proto(stage, app_id, executor_metrics)
+                    if stage_data:
+                        proto_output["stages"].append(stage_data)
+                    else:
+                        failed_stages += 1
+                        logging.debug(f"Stage {stage_id} processing returned None")
+                        
+                except ValueError as e:
+                    failed_stages += 1
+                    stage_id = stage.get('stageId', 'unknown')
+                    error_msg = str(e)
+                    
+                    if "Databricks API returned non-JSON response" in error_msg:
+                        logging.warning(f"Stage {stage_id}: Databricks API issue - {error_msg}")
+                        if args.debug:
+                            logging.debug(f"Stage {stage_id} failed due to Databricks API response issue. This may indicate session timeout or rate limiting.")
+                    elif "Server returned non-JSON response" in error_msg:
+                        logging.warning(f"Stage {stage_id}: Server returned non-JSON response - {error_msg}")
+                        if args.debug:
+                            logging.debug(f"Stage {stage_id} failed due to server returning non-JSON response.")
+                    else:
+                        logging.warning(f"Stage {stage_id}: API error - {error_msg}")
+                        if args.debug:
+                            logging.debug(f"Stage {stage_id} failed due to API error: {error_msg}")
+                    
+                    pbar.update(1)
+                    continue
+                        
+                except Exception as e:
+                    failed_stages += 1
+                    stage_id = stage.get('stageId', 'unknown')
+                    logging.warning(f"Stage {stage_id}: Unexpected error - {str(e)}")
+                    if args.debug:
+                        logging.debug(f"Stage {stage_id} failed due to unexpected error: {str(e)}")
+                        import traceback
+                        logging.debug(f"Exception traceback: {traceback.format_exc()}")
+                
+                pbar.update(1)
+            
+            if args.live_app and batch_idx < total_batches - 1:
+                time.sleep(1)
 
     if failed_stages > 0:
         print(f"⚠️  {failed_stages} stages encountered errors and were skipped")
         logging.info(f"Failed stages for application {app_id}: {failed_stages} out of {len(stages)}")
+    
+    if skipped_stages > 0 and args.live_app:
+        print(f"⚠️  {skipped_stages} stages were skipped (likely removed during live app processing)")
+        logging.info(f"Skipped stages for live application {app_id}: {skipped_stages} out of {len(stages)}")
+    elif skipped_stages > 0:
+        print(f"⚠️  {skipped_stages} stages were skipped due to processing errors")
+        logging.info(f"Skipped stages for application {app_id}: {skipped_stages} out of {len(stages)}")
 
-    if args and getattr(args, 'save_local', False):
+    if os.environ.get('SPARK_ANALYZER_SAVE_LOCAL_MODE') == '1':
         success = save_local_output(proto_output, app_id)
         if success:
             successful_upload_app_ids.append(app_id)
@@ -366,58 +474,23 @@ def run_analysis():
         use_staging = args.env == 'staging'
         s3_uploader = S3Uploader(config_file="config.ini", use_staging=use_staging, args=args)
         
-        if getattr(args, 'save_local', False):
-            import subprocess
-            
+        if os.environ.get('SPARK_ANALYZER_SAVE_LOCAL_MODE') == '1':
             s3_upload_enabled = False
             print("💾 Running in local save mode - analysis data will be saved locally")
-            
-            called_from_config = os.environ.get('SPARK_ANALYZER_SAVE_LOCAL_MODE') == '1'
-            
-            if not called_from_config:
-                print("🔧 Running configuration wizard...")
-                
-                env = os.environ.copy()
-                env['SPARK_ANALYZER_SAVE_LOCAL_MODE'] = '1'
-                
-                original_args = []
-                if args.opt_out:
-                    original_args.extend(['--opt-out', args.opt_out])
-                if args.verbose:
-                    original_args.append('--verbose')
-                if args.debug:
-                    original_args.append('--debug')
-                if args.env != 'production':
-                    original_args.extend(['--env', args.env])
-                if args.cost_estimator_id:
-                    original_args.extend(['--cost-estimator-id', args.cost_estimator_id])
-                if args.server_url:
-                    original_args.extend(['--server-url', args.server_url])
-                if args.app_id:
-                    original_args.extend(['--app-id', args.app_id])
-                if args.stage_id:
-                    original_args.extend(['--stage-id', args.stage_id])
-                if args.upload_url:
-                    original_args.extend(['--upload-url', args.upload_url])
-                if args.local:
-                    original_args.append('--local')
-                if args.browser:
-                    original_args.append('--browser')
-                
-                if original_args:
-                    env['SPARK_ANALYZER_ORIGINAL_ARGS'] = ' '.join(original_args)
-                
-                result = subprocess.run(['spark-analyzer-configure'], check=False, env=env)
-                if result.returncode != 0:
-                    print("❌ Configuration was cancelled or failed. Exiting.")
-                    sys.exit(1)
-                else:
-                    sys.exit(0)
-                
             client = APIClient(args=args)
         else:
             s3_upload_enabled = s3_uploader.is_upload_enabled()
             
+            # Check if --opt-out is used without proper configuration
+            if args.opt_out and not args.cost_estimator_id:
+                print_error_box(
+                    "CONFIGURATION REQUIRED",
+                    "To use --opt-out, please run the configuration wizard first.",
+                    "Run: spark-analyzer-configure --opt-out name,description,details\n\n" +
+                    "This will guide you through the setup process with privacy protection enabled."
+                )
+                sys.exit(1)
+            
             if not s3_upload_enabled:
                 current_env = "staging" if s3_uploader.is_staging_mode() else "production"
                 staging_requested = s3_uploader.was_staging_explicitly_requested()
@@ -647,6 +720,11 @@ def run_analysis():
             failed_upload_app_ids = []
 
             for app_index, app_id in enumerate(app_ids):
+                if app_index > 0:
+                    import time
+                    print("⏳ Waiting 2 seconds before processing next application...")
+                    time.sleep(2)
+                
                 while True:
                     result = process_app_id(app_id, applications, client, spark_stage_analyzer, s3_uploader, args, s3_upload_enabled, processed_app_ids, successful_upload_app_ids, failed_upload_app_ids, summary_app_ids, app_index, len(app_ids))
                     if result == 'success':
@@ -739,7 +817,7 @@ def run_analysis():
                         print("\n❌ Error: Please enter 'y' for yes or 'n' for no.")
 
             # Skip S3 completion signaling if in save-local mode
-            if s3_upload_enabled and successful_upload_app_ids and not getattr(args, 'save_local', False):
+            if s3_upload_enabled and successful_upload_app_ids and not os.environ.get('SPARK_ANALYZER_SAVE_LOCAL_MODE') == '1':
                 print("🔄 Finalizing upload process...")
                 try:
                     if s3_uploader.signal_all_jobs_completion(successful_upload_app_ids):
@@ -761,7 +839,7 @@ def run_analysis():
             for app_id in summary_app_ids:
                 print(f"   - {app_id}")
             print("")
-            if getattr(args, 'save_local', False):
+            if os.environ.get('SPARK_ANALYZER_SAVE_LOCAL_MODE') == '1':
                 if successful_upload_app_ids:
                     print(f"💾 Local Save: Data was successfully saved locally for {len(successful_upload_app_ids)} application(s):")
                     for app_id in successful_upload_app_ids:
