@@ -121,6 +121,15 @@ class APIClient:
         
         if self.base_url.count('/') > 3 and not '/api/v1' in self.base_url:
             logging.info(f"Detected custom path in URL. API endpoints will be appended as {self.base_url}/api/v1/...")
+        
+        self.live_app = False
+        if 'processing' in self.config and 'live_app' in self.config['processing']:
+            try:
+                self.live_app = self.config.getboolean('processing', 'live_app')
+                logging.debug(f"Live app setting from config: {self.live_app}")
+            except (ValueError, configparser.Error):
+                logging.warning("Invalid live_app setting in config, using default: False")
+                self.live_app = False
             
         if args and hasattr(args, 'env') and args.env == 'staging':
             env_var = os.environ.get('SPARK_ANALYZER_STAGING_HISTORY_SERVER_URL')
@@ -129,8 +138,8 @@ class APIClient:
         
         self.session = requests.Session()
         retry_strategy = Retry(
-            total=0,
-            backoff_factor=0.5,
+            total=3,
+            backoff_factor=1.0,
             status_forcelist=[429, 500, 502, 503, 504],
             allowed_methods=["GET"]
         )
@@ -138,7 +147,7 @@ class APIClient:
         self.session.mount("http://", adapter)
         self.session.mount("https://", adapter)
         
-        self.timeout = 3
+        self.timeout = 60 
         
         self.raw_cookies = None
         if args and args.browser:
@@ -283,7 +292,7 @@ class APIClient:
     
     def _request_with_retry(self, method: Callable, url: str, **kwargs) -> requests.Response:
         """Make a request with manual retry logic for cases not covered by the session adapter."""
-        max_manual_retries = 0
+        max_manual_retries = 2 
         retry_delay = 1
         
         for retry_count in range(max_manual_retries + 1):
@@ -296,7 +305,7 @@ class APIClient:
                     time.sleep(retry_delay)
                     retry_delay *= 2
                 else:
-                    logging.error(f"Request to {url} failed without retries: {str(e)}")
+                    logging.error(f"Request to {url} failed after {max_manual_retries} retries: {str(e)}")
                     raise
             except Exception as e:
                 logging.error(f"Request to {url} failed with unexpected error: {str(e)}")
@@ -328,7 +337,8 @@ class APIClient:
         api_base_url = self._ensure_api_path(self.base_url)
         url = f"{api_base_url}{endpoint}"
         
-        logging.debug(f"Constructed full request URL: {url}")
+        logging.debug(f"Making API request to endpoint: {endpoint}")
+        logging.debug(f"Full URL: {url}")
         
         try:
             logging.debug(f"Making request to: {url}")
@@ -363,12 +373,42 @@ class APIClient:
             try:
                 return response.json()
             except json.JSONDecodeError as e:
-                logging.error(f"Invalid JSON response from {url}: {str(e)}")
+                logging.error(f"JSON decode error for {endpoint}: {str(e)}")
+                
+                # Log additional debug information
                 if hasattr(self, 'args') and self.args and hasattr(self.args, 'debug') and self.args.debug:
-                    logging.error(f"Response status code: {response.status_code}")
-                    logging.error(f"Response headers: {dict(response.headers)}")
-                    logging.error(f"Response content: {response.text}")
-                raise
+                    logging.debug(f"Response status code: {response.status_code}")
+                    logging.debug(f"Response headers: {dict(response.headers)}")
+                    logging.debug(f"Response content (first 1000 chars): {response.text[:1000]}")
+                
+                user_url = self.original_url if hasattr(self, 'original_url') else "unknown"
+                
+                if "console.aws.amazon.com" in user_url or "aws.amazon.com" in user_url:
+                    print_error_box(
+                        "AWS CONSOLE URL DETECTED",
+                        f"You entered an AWS console URL: {user_url}",
+                        "1. This is not a Spark History Server URL\n"
+                        "2. You need the Spark History Server URL from your EMR cluster\n"
+                        "3. Find it in the EMR console under your cluster's 'Application history' tab\n"
+                        "4. The URL should look like: http://your-cluster-master:18080"
+                    )
+                    sys.exit(1)
+                elif "databricks.com" in user_url and "sparkui" in user_url:
+                    logging.warning(f"Databricks API returned non-JSON response for {endpoint}. This may indicate session timeout, rate limiting, or server issues.")
+                    raise ValueError(f"Databricks API returned non-JSON response for {endpoint}. Response status: {response.status_code}. This may indicate session timeout, rate limiting, or server issues.")
+                elif "localhost" in user_url or "127.0.0.1" in user_url:
+                    print_error_box(
+                        "LOCAL SPARK HISTORY SERVER ERROR",
+                        f"Could not connect to local Spark History Server: {user_url}",
+                        "1. Make sure Spark History Server is running on your local machine\n"
+                        "2. Check if the port is correct (default is 18080)\n"
+                        "3. Try accessing the URL in your browser first\n"
+                        "4. If using a different port, update your URL accordingly"
+                    )
+                    sys.exit(1)
+                else:
+                    logging.warning(f"Server at {user_url} returned non-JSON response for {endpoint}. Status: {response.status_code}")
+                    raise ValueError(f"Server returned non-JSON response for {endpoint}. Status: {response.status_code}. Response preview: {response.text[:200]}")
             
         except requests.exceptions.ConnectionError as e:
             if hasattr(self, 'args') and self.args and hasattr(self.args, 'debug') and self.args.debug:
@@ -459,7 +499,7 @@ class APIClient:
                     f"You entered an incorrect Databricks Spark UI URL",
                     "1. This is a web UI URL, not an API URL\n"
                     "2. Copy the entire Spark UI URL you see in your browser\n"
-                    "3. Example format: https://dbc-xxx.cloud.databricks.com/compute/sparkui/application-id/driver-id?o=workspace-id\n"
+                    "3. Example format: https://dbc-xxx.cloud.databricks.com/sparkui/application-id/driver-id?o=workspace-id\n"
                     "4. Make sure you have valid Databricks cookies configured"
                 )
             elif "localhost" in user_url or "127.0.0.1" in user_url:
@@ -543,6 +583,48 @@ class APIClient:
             logging.error(f"Failed to get stage details for {app_id}/{stage_id}: {str(e)}")
             raise
     
+    def check_stage_exists(self, app_id: str, stage_id: str) -> bool:
+        logging.debug(f"Checking if stage {stage_id} exists in application {app_id}")
+        try:
+            clean_app_id = app_id.strip()
+            clean_stage_id = str(stage_id).strip()
+            
+            api_base_url = self._ensure_api_path(self.base_url)
+            url = f"{api_base_url}/applications/{clean_app_id}/stages/{clean_stage_id}?details=false"
+            
+            logging.debug(f"Making validation request to: {url}")
+            
+            kwargs = {"timeout": self.timeout}
+            
+            if hasattr(self, 'cookie_dict') and self.cookie_dict:
+                kwargs["cookies"] = self.cookie_dict
+                if "databricks.com" in url:
+                    kwargs["headers"] = {
+                        "Accept": "application/json",
+                        "Content-Type": "application/json",
+                        "User-Agent": "Spark-Analyzer/1.0"
+                    }
+            else:
+                kwargs["headers"] = self.headers if self.headers else {}
+            
+            response = self._request_with_retry(self.session.get, url, **kwargs)
+            response.raise_for_status()
+            
+            # If we get here, the stage exists
+            logging.debug(f"Stage {stage_id} exists (status: {response.status_code})")
+            return True
+            
+        except requests.exceptions.HTTPError as e:
+            if e.response.status_code == 404:
+                logging.debug(f"Stage {stage_id} not found (404)")
+                return False
+            else:
+                logging.debug(f"HTTP error {e.response.status_code} for stage {stage_id}: {e}")
+                return False
+        except Exception as e:
+            logging.debug(f"Error checking stage {stage_id}: {str(e)}")
+            return False
+    
     def get_stage_details_with_tasks(self, app_id: str, stage_id: str) -> Dict[str, Any]:
         """Get details for a specific stage with tasks."""
         logging.debug(f"Fetching details with tasks for stage {stage_id} in application {app_id}")
