@@ -294,9 +294,23 @@ class SparkStageAnalyzer:
                 if not stage_details:
                     logging.debug(f"No stage details found for stage {stage_id}")
                     return None
+            except ValueError as e:
+                error_msg = str(e)
+                if "Databricks API returned non-JSON response" in error_msg:
+                    logging.warning(f"Stage {stage_id}: Databricks API issue - {error_msg}")
+                    return None
+                elif "Server returned non-JSON response" in error_msg:
+                    logging.warning(f"Stage {stage_id}: Server returned non-JSON response - {error_msg}")
+                    return None
+                else:
+                    raise
             except Exception as e:
-                logging.error(f"Error fetching details for stage {stage_id}: {str(e)}")
-                raise ValueError(f"Failed to get stage details: {str(e)}")
+                if "404" in str(e) or "not found" in str(e).lower():
+                    logging.info(f"Stage {stage_id} not found (likely removed), skipping")
+                    return None
+                else:
+                    logging.error(f"Error fetching details for stage {stage_id}: {str(e)}")
+                    raise ValueError(f"Failed to get stage details: {str(e)}")
 
             stage_attempt = stage_details[0] if isinstance(stage_details, list) else stage_details
             
@@ -327,6 +341,18 @@ class SparkStageAnalyzer:
             description = str(stage.get("description", "")) if stage.get("description") is not None else ""
             details = str(stage.get("details", "")) if stage.get("details") is not None else ""
 
+            try:
+                workflow_info = self._get_workflow_info(name, description, stage_id, app_id, details)
+                logging.debug(f"Stage {stage_id} workflow type: {workflow_info['type']}, storage format: {workflow_info['storage_format']}")
+            except Exception as e:
+                logging.error(f"Error determining workflow info for stage {stage_id}: {str(e)}")
+                logging.debug(f"Exception traceback: {traceback.format_exc()}")
+                workflow_info = {
+                    "type": "WORKFLOW_TYPE_UNKNOWN",
+                    "storage_format": "STORAGE_FORMAT_UNKNOWN",
+                    "custom_info": ""
+                }
+
             if "name" in self.opt_out_fields:
                 logging.debug(f"Hashing stage name for privacy (stage {stage_id})")
                 name = f"name_hash_{hash(name)}"
@@ -363,18 +389,6 @@ class SparkStageAnalyzer:
                     executor_run_time_ms = int(executor_run_time_ms)
                 except (ValueError, TypeError):
                     executor_run_time_ms = 0
-            
-            try:
-                workflow_info = self._get_workflow_info(name, description, stage_id, app_id, details)
-                logging.debug(f"Stage {stage_id} workflow type: {workflow_info['type']}, storage format: {workflow_info['storage_format']}")
-            except Exception as e:
-                logging.error(f"Error determining workflow info for stage {stage_id}: {str(e)}")
-                logging.debug(f"Exception traceback: {traceback.format_exc()}")
-                workflow_info = {
-                    "type": "WORKFLOW_TYPE_UNKNOWN",
-                    "storage_format": "STORAGE_FORMAT_UNKNOWN",
-                    "custom_info": ""
-                }
 
             active_executors, active_executor_ids, effective_cores_used, total_cores_available = self._get_active_executors_during_stage(
                 submission_time,
